<?php

declare(strict_types=1);

namespace Drupal\db_queries\StackMiddleware;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Database;
use Drupal\Core\Database\Event\StatementExecutionEndEvent;
use Drupal\Core\Database\Event\StatementExecutionStartEvent;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Start the database logger.
 */
class DbQueriesMiddleware implements HttpKernelInterface {

  /**
   * The decorated kernel.
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $httpKernel;

  /**
   * The admin toolbar tools configuration.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $settings;

  /**
   * Constructs a DbQueriesMiddleware object.
   *
   * @param \Symfony\Component\HttpKernel\HttpKernelInterface $httpKernel
   *   The decorated http kernel.
   * @param \Drupal\Core\Config\Config $config_factory
   *   Config factory.
   */
  public function __construct(HttpKernelInterface $httpKernel, ConfigFactoryInterface $config_factory) {
    $this->httpKernel = $httpKernel;
    $this->settings = $config_factory->get('db_queries.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function handle(Request $request, int $type = self::MAIN_REQUEST, bool $catch = TRUE): Response {

    // Don't start logging database queries unless they have been enabled.
    if ($this->settings->get('profiling_enabled')) {

      \array_map(static function (string $key): void {
        $connection = Database::getConnection($key);
        $connection->enableEvents([
          StatementExecutionStartEvent::class,
          StatementExecutionEndEvent::class,
        ]);
      }, \array_keys(Database::getAllConnectionInfo()));

    }

    return $this->httpKernel->handle($request, $type, $catch);
  }

}
